clear all; clc

% Random samples from mixture of distributions

rng(12345) % set seed

% Generalized Double Exponential

n = 10000; % sample size
p = 1/3; 
lam1 = 1;
lam2 = 3;

x = gdexprnd(n, p, lam1, lam2); % draw sample from gdExp(p, lam1, lam2) 
histogram(x)

% calculate pdf

xvals = -5:0.01:5;
yvals = gdexppdf(xvals, p, lam1, lam2);

% compare results
figure
hold on
histogram(x, 100, 'facecolor', 'red', 'normalization', 'pdf')
plot(xvals, yvals, 'color', 'blue', 'linewidth', 2)
legend('empirical', 'true')
title('$(p, \lambda_{1}, \lambda_2) = (0.5, 1, 1)$', 'interpreter', 'LaTex')
xlabel('x')
ylabel('Density')
hold off

% various plots
x1 = gdexprnd(n, 0.1, 1, 3);
x2 = gdexprnd(n, 0.3, 1, 3);
x3 = gdexprnd(n, 0.5, 1, 3);
xvals1 = min(x1):0.01:max(x1);
xvals2 = min(x2):0.01:max(x2);
xvals3 = min(x3):0.01:max(x3);
yvals1 = gdexppdf(xvals1, 0.1, 1, 3);
yvals2 = gdexppdf(xvals2, 0.3, 1, 3);
yvals3 = gdexppdf(xvals3, 0.5, 1, 3);

figure
subplot(3,1,1)
hold on
histogram(x1, 100, 'facecolor', 'red', 'normalization', 'pdf')
plot(xvals1, yvals1, 'color', 'blue', 'linewidth', 2)
legend('empirical', 'true')
title('$(p, \lambda_{1}, \lambda_2) = (0.1, 1, 3)$', 'interpreter', 'LaTex')
xlabel('x')
ylabel('Density')
hold off
subplot(3,1,2)
hold on
histogram(x2, 100, 'facecolor', 'red', 'normalization', 'pdf')
plot(xvals2, yvals2, 'color', 'blue', 'linewidth', 2)
legend('empirical', 'true')
title('$(p, \lambda_{1}, \lambda_2) = (0.3, 1, 3)$', 'interpreter', 'LaTex')
xlabel('x')
ylabel('Density')
hold off
subplot(3,1,3)
hold on
histogram(x3, 100, 'facecolor', 'red', 'normalization', 'pdf')
plot(xvals3, yvals3, 'color', 'blue', 'linewidth', 2)
legend('empirical', 'true')
title('$(p, \lambda_{1}, \lambda_2) = (0.5, 1, 3)$', 'interpreter', 'LaTex')
xlabel('x')
ylabel('Density')
hold off


% Mixture of Normals

p = [0.50 0.0 0.50]; % mixing coefficients
mus = [-5 0 5]; % Normal means
vars = [1.2 1 0.25]; % Normal variances

xvals = -8:0.01:8; % evaluate pdf
mixnormpdf = @(x) p(1) * normpdf(x, mus(1), sqrt(vars(1))) + ...
    p(2) * normpdf(x, mus(2), sqrt(vars(2))) + ...
    p(3) * normpdf(x, mus(3), sqrt(vars(3)));

yvals = mixnormpdf(xvals);

x = mixnorm(100000, p, mus, vars); % draw sample

% plot results
figure
hold on
histogram(x, 200,'facecolor', 'red', 'normalization', 'pdf')
plot(xvals, yvals, 'color', 'blue', 'linewidth', 2)
% plot(xvals, p(1)*normpdf(xvals,mus(1),sqrt(vars(1))),'color', 'green', 'linewidth', 1)
% plot(xvals, p(2)*normpdf(xvals,mus(2),sqrt(vars(2))),'color', 'yellow', 'linewidth', 1)
% plot(xvals, p(3)*normpdf(xvals,mus(3),sqrt(vars(3))),'color', 'black', 'linewidth', 1)
legend('empirical', 'true')
title('$3$-component Normal Mixture', 'interpreter', 'LaTex')
xlabel('x')
ylabel('Density')
hold off

