clear all; clc

rng(12345) % set seed

% Generate n pairs (X, U) ~ U{(x,u): 0 < u< f(x)}, where U is auxilliary & X ~ Be(a, b).
% Accepted values are random sample from Be(a,b)

n = 100000; % number of pairs

a = 2.7; % Beta parameters
b = 6.3;

xvals = 0.000001:0.00001:1;
yvals = betapdf(xvals, a, b); % evaluate 

[m, mpos] = max(yvals); % value and position of maximum

figure
hold on
plot(xvals, yvals,'color','blue')
yL = get(gca,'YLim');
line([xvals(mpos) xvals(mpos)],yL,'Color','r');
xL = get(gca,'XLim');
line(xL, [m m],'Color','r');
plot(xvals(mpos), m, '*g')
title('Be(2.7, 6.3) pdf')
hold off

m = 1/beta(a, b) * ((a-1)/(a+b-2)) ^ (a - 1) * (1 - ((a-1)/(a+b-2))) ^ (b - 1); % maximum of Beta pdf (as above)

% anonymous functions: no need to use betapdf
%mybetapdf = @(x) 1/beta(a, b) * x .^ (a - 1) .* (1 - x) .^ (b - 1); 

xaccs = []; % empty vector of accepted values
xrejs = []; % empty vector of rejected values 

for i = 1:n
    
    u = [rand unifrnd(0, m)]; % draw uniform sample from the box [0, 1] \times [0, m] 
    
    if u(2) < betapdf(u(1), a, b) % if constraint u_2 < f(u_1) is satisfied
        
        xaccs = [xaccs; u]; % accept the pair
        
    else
        
        xrejs = [xrejs; u]; % reject
        
    end
    
end

figure
hold on
plot(xaccs(:,1), xaccs(:,2), 'o', 'color','green')
plot(xrejs(:,1), xrejs(:,2), 'o', 'color','red')
plot(xvals, yvals,'color', 'blue','LineWidth', 1.2)
ylim([0, 2.675])
xlabel('x','interpreter','latex')
ylabel('Density','interpreter','latex')
title('Accept-Reject sampling $\mathcal{B}(2.7, 6.3)$','interpreter','latex')
hold off

accprob = length(xaccs(:, 1)); % sample acceptance probability

% generate sample from Be(a, b) of size n using function

x = arbeta(200000, a, b);

figure
hold on
histogram(x, 100, 'facecolor', 'blue', 'normalization', 'pdf')
plot(xvals, yvals, 'color', 'red', 'linewidth', 2.5)
legend('empirical', 'true')
hold off

% sample from density known up to normalizing constant

lab4pdf = @(x) exp(-0.5 * x.^2) .* (sin(6*x).^2 + 3*cos(x).^2.*sin(4*x).^2+1);

xvals = -4:1e-04:4;
yvals = lab4pdf(xvals); % evaluate

figure
hold on
plot(xvals, yvals,'color','blue')
plot(xvals, 5*exp(-0.5*xvals.^2),'color','red')
hold off

x = arlab4pdf(250000);

figure
hold on
histogram(x, 100, 'facecolor', 'cyan', 'normalization', 'pdf')
histogram(normrnd(0,1,50000,1), 100, 'facecolor', 'magenta', 'normalization', 'pdf')
hold off


